/**
 * Data Event Constants
 * @module built.core.events.event
 */
define(function(require, exports, module){

    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.event
     */
    exports.CHANGE           = 'change';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.event
     */
    exports.CANCEL           = 'cancel';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.event
     */
    exports.SELECT           = 'select';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.event
     */
    exports.DESELECT         = 'deselect';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.event
     */
    exports.MARKER           = 'marker';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.event
     */
    exports.MARKER_ELEMENT   = 'marker:element';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.event
     */
    exports.SCROLL           = 'scroll';
});
