/**
 * Data Event Constants
 * @module built.core.events.focus
 */
define(function(require, exports, module){

    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.focus
     */
    exports.FOCUS = 'focus:focus';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.focus
     */
    exports.BLUR = 'focus:blur';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.focus
     */
    exports.FOCUS_KEY = 'focus:key';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.focus
     */
    exports.FOCUS_MOUSE = 'focus:mouse';

});
