/**
 * Data Event Constants
 * @module built.core.events.drag
 */
define(function(require, exports, module){

    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.drag
     */
    exports.DRAG_START = 'drag:start';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.drag
     */
    exports.DRAG_END = 'drag:end';
    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.drag
     */
    exports.DRAG_UPDATE = 'drag:update';
});
