@mixin no-select {
    // remove all selection outlines, highlights on elements (mostly affects touch)

    -webkit-touch-callout: none;
    -webkit-user-select: none;
    -khtml-user-select: none;
    -moz-user-select: none;
    -ms-user-select: none;
    user-select: none;
}
