// This Source Code Form is subject to the terms of the Mozilla Public
// License, v. 2.0. If a copy of the MPL was not distributed with this
// file, You can obtain one at http://mozilla.org/MPL/2.0/.
//
// Owner: mark@famo.us
// @license MPL 2.0
// @copyright Famous Industries, Inc. 2014
//

@mixin print-famous() {

    html {
        width: 100%;
        height: 100%;
        margin: 0px;
        padding: 0px;
        overflow: hidden;
        -webkit-transform-style: preserve-3d;
        transform-style: preserve-3d;
    }

    body {
        position: absolute;
        width: 100%;
        height: 100%;
        margin: 0px;
        padding: 0px;
        -webkit-transform-style: preserve-3d;
        transform-style: preserve-3d;
        -webkit-font-smoothing: antialiased;
        -webkit-tap-highlight-color: transparent;
        -webkit-perspective: 0;
        perspective: none;
        overflow: hidden;
    }

    .famous-container, .famous-group {
        position: absolute;
        top: 0px;
        left: 0px;
        bottom: 0px;
        right: 0px;
        overflow: visible;
        -webkit-transform-style: preserve-3d;
        transform-style: preserve-3d;
        -webkit-backface-visibility: visible;
        backface-visibility: visible;
        pointer-events: none;
    }

    .famous-group {
        width: 0px;
        height: 0px;
        margin: 0px;
        padding: 0px;
        -webkit-transform-style: preserve-3d;
        transform-style: preserve-3d;
    }

    .famous-surface {
        position: absolute;
        -webkit-transform-origin: center center;
        transform-origin: center center;
        -webkit-backface-visibility: visible;
        backface-visibility: visible;
        -webkit-transform-style: flat;
        transform-style: preserve-3d; /* performance */
        -webkit-box-sizing: border-box;
        -moz-box-sizing: border-box;
        -webkit-tap-highlight-color: transparent;
        pointer-events: auto;
    }

    .famous-container-group {
        position: relative;
        width: 100%;
        height: 100%;
    }

    .backface-visible{
        -webkit-backface-visibility: visible;
        backface-visibility: visible;
    }
}
