/**
 * Data Event Constants
 * @module built.core.events.data
 */

define(function(require, exports, module){

    /**
     * Data constant
     * @var
     * @type {String}
     * @default
     * @memberOf built.core.events.data
     */
    exports.DATA = 'data:data';
});
