/**
 * Data Event Constants
 * @module built.app.modals.events
 */
define(function(require, exports, module) {

/**
 * Present constant
 * @var
 * @type {String}
 * @default
 * @memberOf built.app.modals.events
 */
exports.PRESENT = 'modal:present';
/**
 * Dismiss constant
 * @var
 * @type {String}
 * @default
 * @memberOf built.app.modals.events
 */
exports.DISMISS = 'modal:dismiss';
/**
 * Complete constant
 * @var
 * @type {String}
 * @default
 * @memberOf built.app.modals.events
 */
exports.COMPLETE = 'modal:complete';

});
